@@ -346,7 +346,7 @@ class domonicTestCase(unittest.TestCase):
         # anchors
 
     def test_dom_querySelector(self):
-        dom1 = html(div(div(div(div(div(div(div( div(_id="thing"), span(_id="fun"), div("asdfasdf", div(), div("yo"), _class="test this thing")))))))))
+        dom1 = html(div(div(div(div(div(div(div(div(_id="thing"), span(_id="fun"), div("asdfasdf", div(), div("yo"), _class="test this thing")))))))))
 
         result = dom1.querySelector('#thing')
         # print('--')
@@ -502,10 +502,10 @@ class domonicTestCase(unittest.TestCase):
                             div(_class="row").html(
                                 div(_class="col-lg-12 text-center").html(
                                     p(
-                                        "Email : ", 
+                                        "Email : ",
                                         a("user@website.com", _href="mailto:user@website.com"),
                                         br(),
-                                        "or Call us on : ", 
+                                        "or Call us on : ",
                                         a("123456789", _href="tel:123456789")
                                     ),
                                     ul(_class="mb-0 list-inline text-center").html(
@@ -616,5 +616,42 @@ class domonicTestCase(unittest.TestCase):
         print('>>>>>>>>>')  # works
 
 
+    def test_dom_decorators(self):
+        from domonic.decorators import el
+
+        @el(html)
+        @el(body)
+        @el(div)
+        def test():
+            return 'hi!'
+        print(test())
+        assert str(test()) == '<html><body><div>hi!</div></body></html>'
+        print('decorators work!')
+
+        @el(html, True)
+        @el(body, True)
+        @el(div, True)
+        def test():
+            return 'hi!'
+        assert test() == '<html><body><div>hi!</div></body></html>'
+        print('decorators work2!')
+
+        @el('html')
+        @el('body')
+        @el('div')
+        def test():
+            return 'hi!'
+        print(test())
+        assert str(test()) == '<html><body><div>hi!</div></body></html>'
+        print('decorators work3!')
+
+        @el(html, True)
+        @el(body)
+        @el('div')
+        def test():
+            return 'hi!'
+        print(test())
+        print('decorators work4!')
+
 if __name__ == '__main__':
     unittest.main()
